#include <windows.h>
#include <direct.h>
#include <stdio.h>

typedef struct {
    char    *title;
    char    *artist;
    char    *album;
    char    *year;
    char    *comment;
    char    *track;
    char    *genre;
} mp3_tag;

// -------------------------------------

void free_tag ( mp3_tag *tag )
{
    if ( tag->title   ) free ( tag->title   );
    if ( tag->artist  ) free ( tag->artist  );
    if ( tag->album   ) free ( tag->album   );
    if ( tag->year    ) free ( tag->year    );
    if ( tag->comment ) free ( tag->comment );
    if ( tag->track   ) free ( tag->track   );
    if ( tag->genre   ) free ( tag->genre   );
}

// -------------------------------------

void get_appdir ( char *app_dir, char **argv )
{
    char drive[_MAX_DRIVE];
    char dir  [_MAX_DIR  ];

    _splitpath ( argv[0], drive, dir, NULL, NULL );
    _makepath  ( app_dir, drive, dir, NULL, NULL );
}

void make_name ( char *buffer, const char *path, const char *name )
{
    char *p = buffer;
    p += sprintf ( p, "%s", path );
    if ( p > buffer && *(p-1) != '\\' ) p += sprintf ( p, "\\" );
    sprintf ( p, "%s", name );
    GetShortPathName ( buffer, buffer, _MAX_PATH );
}

int fgets_clean ( char *buffer, int maxlen, FILE *fp )
{
    int len;
    if ( fgets (buffer, maxlen, fp) == NULL ) return EOF;
    len = strlen ( buffer );
    while ( len > 0 && (buffer[len-1] == '\r' || buffer[len-1] == '\n') ) len--;
    buffer[len] = '\0';
    return len;
}

int load_config ( char **argv, char *mpc_settings, char *mp3_settings, int maxlen )
{
    char drive [_MAX_DRIVE];
    char dir   [_MAX_DIR  ];
    char fname [_MAX_FNAME];
    char config[_MAX_PATH ];
    char str[4096];
    FILE *fp;
    int  line = 0;

    mpc_settings[0] = '\0';
    mp3_settings[0] = '\0';

    _splitpath ( argv[0], drive, dir, fname, NULL   );
    _makepath  ( config,  drive, dir, fname, ".cfg" );

    fp = fopen ( config, "rt" );
    if ( !fp ) return 1;

    while ( fgets_clean (str, sizeof (str), fp) != EOF ) {
        if      ( line == 0 ) {
            strncpy ( mpc_settings, str, maxlen-1 );
            mpc_settings[maxlen-1] = '\0';
        }
        else if ( line == 1 ) {
            strncpy ( mp3_settings, str, maxlen-1 );
            mp3_settings[maxlen-1] = '\0';
        }
        else break;
        line++;
    }

    fclose ( fp );

    return 0;
}

int get_mp3tag ( mp3_tag *tag, int argc, char **argv )
{
    int i;

    for ( i = 1; i < argc; i++ ) {
        if ( !strcmp (argv[i], "--tag") && i+1 < argc ) {
            char *p;
            i++;

            p = strchr ( argv[i], '=' );
            if ( !p || p <= argv[i] ) continue;

            if      ( !strnicmp (argv[i], "title",   p-argv[i]) ) {
                tag->title   = strdup ( p+1 );
            }
            else if ( !strnicmp (argv[i], "artist",  p-argv[i]) ) {
                tag->artist  = strdup ( p+1 );
            }
            else if ( !strnicmp (argv[i], "album",   p-argv[i]) ) {
                tag->album   = strdup ( p+1 );
            }
            else if ( !strnicmp (argv[i], "year",    p-argv[i]) ) {
                tag->year    = strdup ( p+1 );
            }
            else if ( !strnicmp (argv[i], "comment", p-argv[i]) ) {
                tag->comment = strdup ( p+1 );
            }
            else if ( !strnicmp (argv[i], "track",   p-argv[i]) ) {
                tag->track   = strdup ( p+1 );
            }
            else if ( !strnicmp (argv[i], "genre",   p-argv[i]) ) {
                tag->genre   = strdup ( p+1 );
            }
        }
        else if ( i+1 < argc ) {
            char *p = argv[i+1];

            if      ( !strcmp (argv[i], "--title"  ) ) {
                tag->title   = strdup ( p );
            }
            else if ( !strcmp (argv[i], "--artist" ) ) {
                tag->artist  = strdup ( p );
            }
            else if ( !strcmp (argv[i], "--album"  ) ) {
                tag->album   = strdup ( p );
            }
            else if ( !strcmp (argv[i], "--year"   ) ) {
                tag->year    = strdup ( p );
            }
            else if ( !strcmp (argv[i], "--comment") ) {
                tag->comment = strdup ( p );
            }
            else if ( !strcmp (argv[i], "--track"  ) ) {
                tag->track   = strdup ( p );
            }
            else if ( !strcmp (argv[i], "--genre"  ) ) {
                tag->genre   = strdup ( p );
            }
        }
    }

    return 0;
}

void get_names ( char *infile, char *outfile, int argc, char **argv )
{
    char drive[_MAX_DRIVE];
    char dir  [_MAX_DIR  ];
    char fname[_MAX_FNAME];
    char ext  [_MAX_EXT  ];

    infile [0] = '\0';
    outfile[0] = '\0';

    _splitpath ( argv[argc-1], drive, dir, fname, ext );
    if ( !stricmp (ext, ".mpc") || !stricmp (ext, ".mp+") ) {   // output name
        int i;
        for ( i = 1; i < argc; i++ ) {
            char *p = strrchr ( argv[i], '.' );
            if ( !p ) continue;

            if ( !stricmp (p, ".wav") ) {   // input name
                strcpy ( infile, argv[i] );
                break;
            }
        }
        _makepath ( outfile, drive, dir, fname, ".mp3" );
    } else {                                                    // input name
        strcpy ( infile, argv[argc-1] );
        _makepath ( outfile, drive, dir, fname, ".mp3" );
    }

    //_splitpath ( argv[1]     , NULL , NULL, fname, ext  );
    //_makepath  ( logname     , drive, dir , fname, ext  );
}

int main ( int argc, char **argv )
{
    char        app_dir[_MAX_PATH];
    char        mpc_enc[_MAX_PATH];
    char        mp3_enc[_MAX_PATH];
    char        mpc_settings[512];
    char        mp3_settings[512];
    char        infile [_MAX_PATH];
    char        outfile[_MAX_PATH];
    mp3_tag     mp3tag;

    char        cmd[8192];

    if ( argc < 2 ) {
        printf ( "Usage: mpc+mp3 [mpc tags...]\n" );
        return 1;
    }

    memset ( &mp3tag, 0, sizeof (mp3tag) );
    load_config ( argv, mpc_settings, mp3_settings, sizeof (mpc_settings) );

    get_appdir ( app_dir, argv );

    make_name ( mpc_enc, app_dir, "mppenc.exe" );
    make_name ( mp3_enc, app_dir, "lame.exe"   );

    get_mp3tag ( &mp3tag, argc, argv );
    get_names ( infile, outfile, argc, argv );

    { // encode MPC
        char    *p = cmd;
        int     i;

        p += sprintf ( p, "%s", mpc_enc );
        p += sprintf ( p, " %s", mpc_settings );

        for ( i = 1; i < argc; i++ ) {
            p += sprintf ( p, " \"%s\"", argv[i] );
        }

        system ( cmd );
    }

    { // encode MP3
        char    *p = cmd;

        p += sprintf ( p, "%s", mp3_enc );
        p += sprintf ( p, " %s", mp3_settings );

        if ( mp3tag.title   ) p += sprintf ( p, " --tt \"%s\"", mp3tag.title   );
        if ( mp3tag.artist  ) p += sprintf ( p, " --ta \"%s\"", mp3tag.artist  );
        if ( mp3tag.album   ) p += sprintf ( p, " --tl \"%s\"", mp3tag.album   );
        if ( mp3tag.year    ) p += sprintf ( p, " --ty \"%s\"", mp3tag.year    );
        if ( mp3tag.comment ) p += sprintf ( p, " --tc \"%s\"", mp3tag.comment );
        if ( mp3tag.track   ) p += sprintf ( p, " --tn \"%s\"", mp3tag.track   );
        if ( mp3tag.genre   ) p += sprintf ( p, " --tg \"%s\"", mp3tag.genre   );

        p += sprintf ( p, " \"%s\"", infile  );
        p += sprintf ( p, " \"%s\"", outfile );

        system ( cmd );
    }

    free_tag ( &mp3tag );

    return 0;
}
